﻿using ProjectGarage.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectGarage.Forms
{
    public partial class FormTrucks : Form
    {
        private readonly IUnityContainer _container;

        private readonly ITruckRepository _truckRepository;

        public FormTrucks(IUnityContainer container, ITruckRepository truckRepository)
        {
            InitializeComponent();
            _container = container ?? throw new ArgumentNullException(nameof(container));
            _truckRepository = truckRepository ?? throw new ArgumentNullException(nameof(truckRepository));
        }
        private void FormTrucks_Load(object sender, EventArgs e)
        {
            try
            {
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при загрузке", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonAddTruck_Click(object sender, EventArgs e)
        {
            try
            {
                _container.Resolve<FormTruck>().ShowDialog();
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при добавлении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonDeleteTruck_Click(object sender, EventArgs e)
        {
            if (!TryGetIDFromSelectedRow(out var findid))
            {
                return;
            }

            if (MessageBox.Show("Удалить запись?", "Удаление", MessageBoxButtons.YesNo) != DialogResult.Yes)
            {
                return;
            }

            try
            {
                _truckRepository.DeleteTruck(findid);
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при удалении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonUpdateTruck_Click(object sender, EventArgs e)
        {
            if (!TryGetIDFromSelectedRow(out var findid))
            {
                return;
            }

            try
            {
                var form = _container.Resolve<FormTruck>();
                form.Id = findid;
                form.ShowDialog();
                LoadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при изменении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void LoadList() => dataGridViewTrucks.DataSource = _truckRepository.ReadTrucks();

        private bool TryGetIDFromSelectedRow(out int id)
        {
            id = 0;
            if (dataGridViewTrucks.SelectedRows.Count < 1)
            {
                MessageBox.Show("Нет выбранной записи", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            id = Convert.ToInt32(dataGridViewTrucks.SelectedRows[0].Cells["Id"].Value);
            return true;
        }
    }
}
