using ProjectGarage.Repositories;
using ProjectGarage.Repositories.Implementations;
using Unity.Lifetime;
using Unity;
using Unity.Microsoft.Logging;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Serilog;

namespace ProjectGarage
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            Application.Run(CreateContainer().Resolve<FormGarage>());
        }

        private static IUnityContainer CreateContainer()
        {
            var container = new UnityContainer();
            container.AddExtension(new LoggingExtension(CreateLoggerFactory()));

            container.RegisterType<ITruckRepository, TruckRepository>();
            container.RegisterType<IFuelRepository, FuelRepository>();
            container.RegisterType<IRouteRepository, RouteRepository>();
            container.RegisterType<IDriverRepository, DriverRepository>();
            container.RegisterType<ITransportationRepository, TransportationRepository>();
            container.RegisterType<IReplenishmentRepository, ReplenishmentRepository>();

            container.RegisterType<IConnectionString, ConnectionString>();

            return container;
        }

        private static LoggerFactory CreateLoggerFactory()
        {
            var loggerFactory = new LoggerFactory();
            loggerFactory.AddSerilog(new LoggerConfiguration()
                .ReadFrom.Configuration(new ConfigurationBuilder()
                .SetBasePath(Directory.GetCurrentDirectory())
                .AddJsonFile("appsettings.json")
                .Build())
                .CreateLogger());
            return loggerFactory;
        }
    }
}