﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectGarage.Entities;
using ProjectGarage.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectGarage.Repositories.Implementations;

public class FuelRepository : IFuelRepository
{
    public readonly IConnectionString _connectionString;

    private readonly ILogger<FuelRepository> _logger;

    public FuelRepository(IConnectionString connectionString, ILogger<FuelRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateFuel(Fuel fuel)
    {
        _logger.LogInformation("Добавление объект");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(fuel));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
INSERT INTO fuel (FuelName, FuelType, Price) VALUES
(@FuelName, @FuelType, @Price);";
            connection.Execute(queryInsert, fuel);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateFuel(Fuel fuel)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(fuel));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
UPDATE fuel
SET
    FuelName=@FuelName,
    FuelType=@FuelType,
    Price=@Price
WHERE Id=@Id";
            connection.Execute(queryUpdate, fuel);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeleteFuel(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
DELETE FROM fuel
WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public Fuel ReadFuelByID(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
SELECT * FROM fuel
WHERE Id=@id";
            var fuel = connection.QueryFirst<Fuel>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(fuel));

            return fuel;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public IEnumerable<Fuel> ReadFuels()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM fuel";
            var fuels = connection.Query<Fuel>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(fuels));
            return fuels;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
