﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectGarage.Entities;
using ProjectGarage.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectGarage.Repositories.Implementations;

public class TruckRepository : ITruckRepository
{
    public readonly IConnectionString _connectionString;

    private readonly ILogger<TruckRepository> _logger;

    public TruckRepository(IConnectionString connectionString, ILogger<TruckRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreateTruck(Truck truck)
    {
        _logger.LogInformation("Добавление объект");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(truck));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
INSERT INTO truck (Numbers, Truck_Type, MaxFuel) VALUES
(@Numbers, @Truck_Type, @MaxFuel);";
            connection.Execute(queryInsert, truck);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateTruck(Truck truck)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(truck));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
UPDATE truck
SET
Numbers=@Numbers,
Truck_Type=@Truck_Type,
MaxFuel=@MaxFuel
WHERE Id=@Id";
            connection.Execute(queryUpdate, truck);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeleteTruck(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
DELETE FROM truck
WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public Truck ReadTruckByID(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
SELECT * FROM truck
WHERE Id=@id";
            var truck = connection.QueryFirst<Truck>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(truck));

            return truck;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public IEnumerable<Truck> ReadTrucks()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM truck";
            var trucks = connection.Query<Truck>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(trucks));
            return trucks;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
